@@ -6,8 +6,9 @@
     Copyright 2009, Alan Calvert
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of version 2 of the GNU General Public
-    License as published by the Free Software Foundation.
+    it and/or modify it under the terms of the GNU Library General Public
+    License as published by the Free Software Foundation; either version 2 of
+    the License, or (at your option) any later version.
 
     yoshimi is distributed in the hope that it will be useful, but WITHOUT ANY
     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -34,14 +35,13 @@ class EffectLFO
         unsigned char Pfreq;
         unsigned char Prandomness;
         unsigned char PLFOtype;
-        unsigned char Pstereo; // "64"=0
+        unsigned char Pstereo; // 64 = centre
     private:
         float getlfoshape(float x);
 
         float xl,xr;
         float incx;
         float ampl1, ampl2, ampr1, ampr2; // necessary for "randomness"
-        float lfointensity;
         float lfornd;
         char lfotype;
 };
